/*
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * com.ibm.rational.wvcm.stp.cc.CcMergeElement
 *
 * (C) Copyright IBM Corporation 2010, 2011.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.wvcm.stp.cc;

import javax.wvcm.Feedback;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stp.cc.CcFile.CcCheckoutFlag;

/**
 * An interface used to track the status of a Merge operation. This object
 * is used by integration operations to maintain state through each phase of
 * the operation. For example, from <code>CcView.doStartDeliver()</code> to
 * <code>CcView.doCompleteDeliver()</code>.
 */
public interface CcMergeElement {

    /**
     * Enumerated states to determine if a merge is needed.
     */
    enum MergeState {
        UNKNOWN,
        NEEDS_MERGE,
        MERGE_NOT_NEEDED,
        MERGED,
        NO_MERGE,
        NEEDS_NOAUTOMERGE,
        MERGE_ERROR,
        NEVER_MERGE
    }

    /**
     * Before the merge stage, the MergeInterventionType will either be 
     * <code>TRIVIAL</code> or <code>NON_TRIVIAL</code>. After the merge stage,
     * MergeInterventionType shows what type of merge was performed, 
     * <code>AUTOMATIC</code> or <code>MANUAL</code>.
     */
    enum MergeInterventionType {
        UNKNOWN,
        /**
         * No user intervention is required.
         */
        TRIVIAL,
        /**
         * User intervention is required.
         */
        NON_TRIVIAL,
        /**
         * No intervention during the merge. Decision was made automatically.
         */
        AUTOMATIC,
        /**
         * There was user intervention during the merge.
         */
        MANUAL
    }

    /**
     * Retrieve whether a merge is needed or not
     *
     * @return MergeState describing the element merge
     */
    public MergeState getMergeState();

    /**
     * Retrieve the view relative pathname for this element
     *
     * @return String pathname to this object
     */
    public String getViewRelativePathname() throws WvcmException;

    /**
     * Is this element a directory?
     *
     * @return true if the element is a directory
     */
    public boolean getIsDirectory();

    /**
     * Determine whether this element is visible to the current view.
     *
     * @return true if the element is visible
     */
    public boolean getIsVisible();

    /**
     * Determine the file type for this element. This is the file type
     * used to determine which type manager to use for the merge handler.
     *
     * @return String of file type (ie. text-delta)
     */
    public String getTypeManager();

    /**
     * Provides an accessor to the underlying CcFile object for this
     * MergeElement. Use this method to request properties.
     * @param wantedProps props to be fetched
     *
     * @return a new proxy for the underlying CcFile
     */
    public CcFile doReadProperties(Feedback wantedProps) throws WvcmException;

    /**
     * Returns the type of merge needed for this element. This
     * will reveal if this element is marked as a USER merge or
     * NEVER merge type.
     *
     * @return ElementMergeType
     */
    public CcElementType.MergeType getElementMergeType();

    /**
     * Returns the type of merge needed.
     * <ul>
     * <li>TRIVIAL - No intervention is needed.</li>
     * <li>NON-TRIVIAL - User intervention is required.</li>
     * <li>AUTOMATIC - No intervention during the merge. Decision was made
     * automatically.</li>
     * <li>MANUAL - There was user intervention during the merge.</li>
     * </ul>
     * @see MergeInterventionType
     *
     * @return MergeInterventionType describing the type of merge
     */
    public MergeInterventionType getMergeInterventionType();

    /**
     * Retrieve the from version contributor.
     *
     * @return String version extended pathname of the from version
     */
    public String getFromVersionExtendedPathname();

    /**
     * Retrieve the to version contributor.
     *
     * @return String version extended pathname of the to version
     */
    public String getToVersionExtendedPathname();

    /**
     * Retrieve the base version contributor. May be null.
     *
     * @return String version extended pathname of the base version
     */
    public String getBaseVersionExtendedPathname();

    /**
     * Get the provider proxy for the underlying CcFile for this element
     *
     * @return CcProvider proxy
     */
    public CcProvider getProvider();

    /**
     * Utility method to request the underlying CcFile object to be
     * checked out for merge.
     *
     * @param coFlags Checkout flags
     * @param feedback properties to be requested
     * @return new proxy to the underlying file with requested properties
     * @throws WvcmException
     */
    public CcFile doCheckout(CcCheckoutFlag[] coFlags, Feedback feedback) throws WvcmException;

}
